function part=partitioner(md,varargin)
%PARTITIONER - partition mesh 
%
%   List of options to partitioner: 
%
%   package: 'chaco', 'metis' or 'scotch'
%   npart: number of partitions.
%   weighting: 'on' or 'off': default off
%   section:  1 by defaults(1=bisection, 2=quadrisection, 3=octasection)
%   
%   Usage:
%      md.part=partitioner(md,'package','chaco','npart',100,'weighting','on');
%


%get options: 
options=pairoptions(varargin{:});

%set defaults
options=addfielddefault(options,'package','chaco');
options=addfielddefault(options,'npart',10);
options=addfielddefault(options,'weighting','on');
options=addfielddefault(options,'section',1);

%get package: 
package=getfieldvalue(options,'package');
npart=getfieldvalue(options,'npart');

if strcmpi(package,'chaco'),

	%  create adjacency matrix, vertex list, and vertex weights
	if isempty(md.adjacency)
		[md.adjacency,vlist,md.vwgt]=adjacency_matrix(md.elements,[md.x md.y md.z]);
	end

	%  default method (from chaco.m)
	method=[1 1 0 0 1 1 50 0 .001 7654321]';
	method(1)=3;    %  global method (3=inertial (geometric))
	method(3)=1;    %  vertex weights (0=off, 1=on)
	
	%specify bisection
	method(6)=getfieldvalue(options,'section');%  ndims (1=bisection, 2=quadrisection, 3=octasection)

	%are we using weights? 
	if strcmpi(getfieldvalue(options,'weighting'),'on'),
	
		%  add integer scaled vertex weights into adjacency matrix for chaco (on diagonal)
%		adj_mat2=md.adjacency+sparse(1:size(md.adjacency,1),1:size(md.adjacency,2),floor(md.vwgt/min(md.vwgt)));
	
		%  partition into nparts
		part=Chaco(md.adjacency,floor(md.vwgt/min(md.vwgt)),[],md.x, md.y ,md.z,method,npart,[])'; 
	else
		%  partition into nparts
		method(3)=0;    %  vertex weights (0=off, 1=on)
		part=Chaco(md.adjacency,[],[],md.x, md.y, md.z,method,npart,[])'; 
	end
	part=part+1; %index partitions from 1 up. like metis.

elseif strcmpi(package,'scotch'),

	%  create adjacency matrix, vertex list, and vertex weights
	if isempty(md.adjacency)
		[md.adjacency,vlist,md.vwgt]=adjacency_matrix(md.elements,[md.x md.y md.z]);
	end

	%  partition into 100 parts, but ignore vertex and edge weights
	%maptab=gmap(md.adjacency,[],[],[],'cmplt',[npart],'-vm','-vs','-vt');
	
	if strcmpi(getfieldvalue(options,'weighting'),'on'),
		maptab=gmap(md.adjacency,[],floor(md.vwgt/min(md.vwgt)),[],'cmplt',[npart]);
	else
		maptab=gmap(md.adjacency,[],[],[],'cmplt',[npart]);
	end

	part=maptab(:,2)+1;

elseif strcmpi(package,'linear'),

	part=1:1:md.numberofgrids;

elseif strcmpi(package,'metis'),

	[element_partitioning,part]=MeshPartition(md,npart);

else

	error(['partitioner error message: could not find ' package ' partitioner']);
	help partitioner

end
